import 'dotenv/config';
import { MatrixClient, MessageEvent, SimpleFsStorageProvider } from 'matrix-bot-sdk';
import { PrismaClient } from '@prisma/client';
import { readFile } from 'fs/promises';
import { join } from 'path';

declare global {
  namespace NodeJS {
    interface ProcessEnv {
      MASTODON_ACCESS_TOKEN: string;
      MASTODON_HOMESERVER_URL: string;
      MATRIX_HOMESERVER_URL: string;
      MATRIX_ACCESS_TOKEN: string;
      MATRIX_ROOM_ID: string;
    }
  }
}

function parseAsyncRegexFile(file: Buffer): RegExp[] {
  const data = file.toString();
  return data
    .split('\n')
    .filter(regex => !regex.startsWith('#') && regex.trim() !== '')
    .map(regex => new RegExp(regex));
}

async function main() {
  const mastodon = await (
    await import('masto')
  ).login({
    url: process.env.MASTODON_HOMESERVER_URL,
    accessToken: process.env.MASTODON_ACCESS_TOKEN,
    disableVersionCheck: true
  });
  const matrix = new MatrixClient(
    process.env.MATRIX_HOMESERVER_URL,
    process.env.MATRIX_ACCESS_TOKEN,
    new SimpleFsStorageProvider('matrix_db.json')
  );
  const nicknameRegexes = parseAsyncRegexFile(await readFile(join(__dirname, '..', 'domains.txt')));
  const domainRegexes = parseAsyncRegexFile(await readFile(join(__dirname, '..', 'nicknames.txt')));
  const db = new PrismaClient();
  const ws = await mastodon.v1.stream.streamUser();
  ws.on('notification', async notification => {
    console.log('tests');
    if (
      nicknameRegexes.some(regex => regex.test(notification.account.displayName)) ||
      nicknameRegexes.some(regex => regex.test(notification.account.acct.split('@').at(0) || '')) ||
      domainRegexes.some(regex => regex.test(notification.account.acct.split('@').at(-1) || ''))
    ) {
      await mastodon.v1.accounts.block(notification.account.id);
      await matrix.sendText(
        process.env.MATRIX_ROOM_ID,
        `Юзер ${notification.account.acct} був заблокований автоматично`
      );
    }

    if (
      notification.type === 'mention' &&
      !notification.status?.inReplyToId &&
      !notification.status?.inReplyToAccountId &&
      notification.status?.visibility === 'public'
    ) {
      const matrixId = await matrix.sendText(
        process.env.MATRIX_ROOM_ID,
        `Одобряємо?\n${notification.status.url}\nт - Так\nн - Ні`
      );
      await db.post.create({
        data: {
          postId: notification.status.id,
          matrixId
        }
      });
    }
    if (notification.type === 'follow') {
      await mastodon.v1.statuses.create({
        visibility: 'public',
        status: `Вітаємо у нашій спільноті! @${notification.account.acct}`
      });
      console.log('follow message');
    }
  });
  matrix.on('room.message', async (roomId, event) => {
    if (
      roomId === process.env.MATRIX_ROOM_ID &&
      event['content'] &&
      ['т', 'н'].includes(event['content']['body'].split('\n').at(-1).trim()) &&
      event['content']['m.relates_to'] &&
      event['content']['m.relates_to']['m.in_reply_to']
    ) {
      const post = await db.post.findUnique({
        where: {
          matrixId: event['content']['m.relates_to']['m.in_reply_to']['event_id']
        }
      });
      if (post && event['content']['body'].split('\n').at(-1) === 'т') {
        console.log('test');
        await mastodon.v1.statuses.reblog(post.postId);
        await matrix.replyText(
          process.env.MATRIX_ROOM_ID,
          event['content']['m.relates_to']['m.in_reply_to']['event_id'],
          'Пост успішно опубліковано'
        );
      }
      if (post && event['content']['body'].split('\n').at(-1) === 'н') {
        await matrix.redactEvent(
          process.env.MATRIX_ROOM_ID,
          event['content']['m.relates_to']['event_id'],
          'Не одобрили :('
        );
        await db.post.delete({
          where: {
            matrixId: event['content']['m.relates_to']['m.in_reply_to']['event_id']
          }
        });
        await matrix.replyText(
          process.env.MATRIX_ROOM_ID,
          event['content']['m.relates_to']['m.in_reply_to']['event_id'],
          'Пост відхилено'
        );
      }
    }
  });
  await db.$connect();
  await matrix.start();
}
main().then(() => console.log('everything started!'));
